﻿/*
 * Copyright (C) 2019 Tianjin KYLIN Information Technology Co., Ltd.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU  Lesser General Public License as published by
 * the Free Software Foundation; either version 2.1, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, see <http://www.gnu.org/licenses/&gt;.
 *
 */
#include "frmlunarcalendarwidget.h"
#include "ui_frmlunarcalendarwidget.h"
#include <QPainter>
#include <KWindowEffects>
#include "morelabel.h"
#define TRANSPARENCY_SETTINGS       "org.ukui.control-center.personalise"
#define TRANSPARENCY_KEY            "transparency"
#define PANEL_CONTROL_IN_CALENDAR "org.ukui.control-center.panel.plugins"
#define LUNAR_KEY "calendar"
#define FIRST_DAY_KEY "firstday"
#include <QDBusInterface>
#include <QScreen>

const int CALENDAR_WIDTH = 440;
const int CALENDAR_HEIGHT = 780;
const int CALENDAR_LUNAR_HEIGHT = 75;
const int CALENDAR_LUNAR_HEIGHT_PLUSE = 77; //852
const int SCHEDULE_ITEM = 30;

#define PANEL_SETTINGS      "org.ukui.panel.settings"
#define PANEL_SIZE_KEY      "panelsize"
#define PANEL_POSITION_KEY  "panelposition"

frmLunarCalendarWidget::frmLunarCalendarWidget(QWidget *parent) : QWidget(parent), ui(new Ui::frmLunarCalendarWidget)
{
    installEventFilter(this);
    ui->setupUi(this);
    connect(ui->lunarCalendarWidget,&LunarCalendarWidget::almanacChanged,this,&frmLunarCalendarWidget::showAlmanac);
    connect(this, &frmLunarCalendarWidget::onShowToday,ui->lunarCalendarWidget,&LunarCalendarWidget::showToday);
    this->initForm();
    //默认展开状态
    bool showLunar = ui->lunarCalendarWidget->getShowLunar();
    if(showLunar)
        this->setFixedSize(CALENDAR_WIDTH, CALENDAR_HEIGHT+CALENDAR_LUNAR_HEIGHT+CALENDAR_LUNAR_HEIGHT_PLUSE);
    else
        this->setFixedSize(CALENDAR_WIDTH,CALENDAR_HEIGHT);

    const QByteArray transparency_id(TRANSPARENCY_SETTINGS);
    if(QGSettings::isSchemaInstalled(transparency_id)){
        transparency_gsettings = new QGSettings(transparency_id);
    }
    const QByteArray calendar_id(PANEL_CONTROL_IN_CALENDAR);
    if(QGSettings::isSchemaInstalled(calendar_id)){
        calendar_gsettings = new QGSettings(calendar_id);
        //公历/农历切换
        connect(calendar_gsettings, &QGSettings::changed, this, [=] (const QString &key){
            if(key == LUNAR_KEY){
                ckShowLunar_stateChanged(calendar_gsettings->get(LUNAR_KEY).toString() == "lunar");
            }
            if (key == FIRST_DAY_KEY) {
                cboxWeekNameFormat_currentIndexChanged(calendar_gsettings->get(FIRST_DAY_KEY).toString() == "sunday");
            }
        });
    } else {
        ckShowLunar_stateChanged(false);
        cboxWeekNameFormat_currentIndexChanged(false);
    }
    setProperty("useStyleWindowManager",false);
    initPanelGSettings();

    QDBusConnection sessionBus = QDBusConnection::sessionBus();
    if (sessionBus.registerService("org.ukui.kylin_calendar")) {
        sessionBus.registerObject("/org/ukui/kylin_calendar", this, QDBusConnection::ExportAllContents);

        qDebug() << "初始化DBUS成功";
    }else
        qDebug() << "初始化DBUS失败";

    QDBusConnection::sessionBus().connect(
        "activate", // 发送者的服务名，留空表示任何服务
        "/org/ukui/kylin_calendar", // 发送者的对象路径
        "org.ukui.kylin_calendar", // 发送者的接口名
        "command_Control", // 方法名
        this, // 接收者
        SLOT(onActivateRequest()) // 槽函数
    );

    if (!m_argName.isEmpty()) {
        qDebug() << "单例参数argName = " << m_argName;
        int num = m_argName.size();
        switch (num) {
        case 1:
            if(QFileInfo::exists(m_argName[0])){   // 不支持用录音打开第三方文件
                QMessageBox::warning(this, tr("Alert"), tr("Recorder does not support selecting the opening method to open files!"));
                break;
            }else{
                command_Control(m_argName[0]);
                break;
            }
        default:
            break;
        }
    }
    m_isFirstObject = false; //可以接收外部命令
}

frmLunarCalendarWidget::~frmLunarCalendarWidget()
{
    delete ui;
}

void frmLunarCalendarWidget::initPanelGSettings()
{
    const QByteArray id(PANEL_SETTINGS);
    if (QGSettings::isSchemaInstalled(id)) {
        if (m_panelGSettings == nullptr) {
            m_panelGSettings = new QGSettings(id);
        }
        if (m_panelGSettings->keys().contains(PANEL_POSITION_KEY)) {
            m_panelPosition = m_panelGSettings->get(PANEL_POSITION_KEY).toInt();
        }
        if (m_panelGSettings->keys().contains(PANEL_SIZE_KEY)) {
            m_panelSize = m_panelGSettings->get(PANEL_SIZE_KEY).toInt();
        }
        connect(m_panelGSettings, &QGSettings::changed, this, [&] (const QString &key) {
            if (key == PANEL_POSITION_KEY) {
                m_panelPosition = m_panelGSettings->get(PANEL_POSITION_KEY).toInt();
            }
            if (key == PANEL_SIZE_KEY) {
                m_panelSize = m_panelGSettings->get(PANEL_SIZE_KEY).toInt();
            }
        });
    }
}

void frmLunarCalendarWidget::onActivateRequest()
{
    show();
    raise();
    activateWindow();
}

void frmLunarCalendarWidget::checkSingle(QStringList path) //检查单例模式
{
    QString str;
    if (path.size() > 1) {
        str = path[1];
    }

    QStringList homePath = QStandardPaths::standardLocations(QStandardPaths::HomeLocation);
    //兼容VNC的单例模式
    QString lockPath = homePath.at(0) + "/.config/kylin-calendar-lock";
    int fd = open(lockPath.toUtf8().data(), O_WRONLY | O_CREAT | O_TRUNC, S_IRUSR | S_IWUSR);
    if (fd < 0) {
        exit(1);
    }
    if (lockf(fd, F_TLOCK, 0)) {
        QDBusInterface interface("org.ukui.kylin_calendar", "/org/ukui/kylin_calendar", "org.ukui.kylin_calendar",
                                         QDBusConnection::sessionBus());
        if (path.size() == 1)
            interface.call("command_Control", str);
        if (path.size() == 2)
            interface.call("command_Control", str);
        qDebug() << "Can't lock single file, kylin-calendar is already running!";
        exit(0);
    }
    m_isFirstObject = true; //我是首个对象
    m_argName << str;
}


void frmLunarCalendarWidget::changeEvent(QEvent *event)
{
    if (event->type() == QEvent::WindowStateChange) {
        auto windowState = window()->windowState();
        if (windowState & Qt::WindowMinimized) {
            // 隐藏窗口至托盘
            showMinimized();
        }
    }
    QWidget::changeEvent(event);
}

int frmLunarCalendarWidget::command_Control(QString cmd1)
{
    qDebug() << "命令:" << cmd1;
    // 990需求命令行控制
    if (cmd1 == "") //无参数，单例触发
    {
        // kwin接口唤醒
        KWindowSystem::forceActiveWindow(this->winId());
        qDebug() << "窗口置顶";
        return 0;
    }else
    if (cmd1 == "-s" || cmd1 == "-show") //打开日历
    {
        show();
        return 0;
    }else
    if (cmd1 == "-h" || cmd1 == "-hide") //隐藏
    {
        hide();
        return 0;
    }else{
        QMessageBox::warning(this, tr("Alert"), tr("Calendar does not support opening file in open mode."));
    }
    return 0; //重要:int类型的函数一定要加返回值
}

/* 显示新版日历窗口 */
QRect frmLunarCalendarWidget::changepos()
{
    QRect availableGeo = QGuiApplication::screenAt(QCursor::pos())->geometry();
    int x, y;
    int margin = 8;

    switch (m_panelPosition)
    {
    case PanelPosition::Top:
        x = availableGeo.x() + availableGeo.width() - this->width() - margin;
        y = availableGeo.y() + m_panelSize + margin;
        break;
    case PanelPosition::Bottom:
        x = availableGeo.x() + availableGeo.width() - this->width() - margin;
        y = availableGeo.y() + availableGeo.height() - m_panelSize - this->height() - margin;
        break;
    case PanelPosition::Left:
        x = availableGeo.x() + m_panelSize + margin;
        y = availableGeo.y() + availableGeo.height() - this->height() - margin;
        break;
    case PanelPosition::Right:
        x = availableGeo.x() + availableGeo.width() - m_panelSize - this->width() - margin;
        y = availableGeo.y() + availableGeo.height() - this->height() - margin;
        break;
    }
   return QRect(x, y, this->width(), this->height());
}

void frmLunarCalendarWidget::showAlmanac(bool big)
{
    qDebug() << __FILE__ <<__LINE__<<__FUNCTION__<<big;
    if(big){
        setFixedHeight(CALENDAR_HEIGHT+CALENDAR_LUNAR_HEIGHT+CALENDAR_LUNAR_HEIGHT_PLUSE);
    }else{
        setFixedHeight(CALENDAR_HEIGHT);
    }
}
void frmLunarCalendarWidget::changeUpSize(int num)
{
    qDebug()<<"----------changeUpsieze";
    yijiIsUp = true;
    this->setFixedSize(CALENDAR_WIDTH, 1050);
    qDebug()<<"changeUpSize里面的宽高"<<CALENDAR_WIDTH<<CALENDAR_HEIGHT+CALENDAR_LUNAR_HEIGHT+CALENDAR_LUNAR_HEIGHT_PLUSE+2*40;
    Q_EMIT yijiFChangeUp();
}

void frmLunarCalendarWidget::changeUp(int n)
{
    if(yijiIsUp){
       this->setFixedSize(CALENDAR_WIDTH, CALENDAR_HEIGHT+CALENDAR_LUNAR_HEIGHT+CALENDAR_LUNAR_HEIGHT_PLUSE+40*n);
    }
    else{
       this->setFixedSize(CALENDAR_WIDTH, CALENDAR_HEIGHT+CALENDAR_LUNAR_HEIGHT+40*n);
    }
    Q_EMIT Up(n);

}

void frmLunarCalendarWidget::changeDownSize(int num)
{
    yijiIsUp = false;
    this->setFixedSize(CALENDAR_WIDTH, 925);
    qDebug()<<"changeDownSize"<<CALENDAR_WIDTH, CALENDAR_HEIGHT+CALENDAR_LUNAR_HEIGHT+2*40;
    Q_EMIT yijiFChangeDown();
}

void frmLunarCalendarWidget::initForm()
{
    //ui->cboxWeekNameFormat->setCurrentIndex(0);
}

void frmLunarCalendarWidget::cboxCalendarStyle_currentIndexChanged(int index)
{
    ui->lunarCalendarWidget->setCalendarStyle((LunarCalendarWidget::CalendarStyle)index);
}

void frmLunarCalendarWidget::cboxSelectType_currentIndexChanged(int index)
{
    ui->lunarCalendarWidget->setSelectType((LunarCalendarWidget::SelectType)index);
}

void frmLunarCalendarWidget::cboxWeekNameFormat_currentIndexChanged(bool FirstDayisSun)
{
    ui->lunarCalendarWidget->setWeekNameFormat(FirstDayisSun);
}

void frmLunarCalendarWidget::ckShowLunar_stateChanged(bool arg1)
{
    if (arg1) {
        if (yijiIsUp) {
            this->setFixedSize(CALENDAR_WIDTH,CALENDAR_HEIGHT+CALENDAR_LUNAR_HEIGHT+CALENDAR_LUNAR_HEIGHT_PLUSE);
        } else {
            this->setFixedSize(CALENDAR_WIDTH,CALENDAR_HEIGHT+CALENDAR_LUNAR_HEIGHT);
        }
    } else {
        this->setFixedSize(CALENDAR_WIDTH,CALENDAR_HEIGHT);
    }
}

void frmLunarCalendarWidget::paintEvent(QPaintEvent *)
{
    QStyleOption opt;
    opt.init(this);
    QRect rect = this->rect();
    QPainter p(this);
    double tran =1;
    const QByteArray transparency_id(TRANSPARENCY_SETTINGS);
    if(QGSettings::isSchemaInstalled(transparency_id)){
       tran=transparency_gsettings->get(TRANSPARENCY_KEY).toDouble()*255;
    }

    QColor color = palette().color(QPalette::Base);
    color.setAlpha(tran);
    QBrush brush =QBrush(color);
    p.setBrush(brush);
    p.setPen(Qt::NoPen);
    p.setRenderHint(QPainter::Antialiasing);
    p.drawRoundedRect(rect,6,6);
    style()->drawPrimitive(QStyle::PE_Widget, &opt, &p, this);
}

/*
 * 事件过滤，检测鼠标点击外部活动区域则收回收纳栏
*/
bool frmLunarCalendarWidget::eventFilter(QObject *obj, QEvent *event)
{
    if (obj == this)
    {
        if (event->type() == QEvent::MouseButtonPress)
           {
               QMouseEvent *mouseEvent = static_cast<QMouseEvent *>(event);
               if (mouseEvent->button() == Qt::LeftButton)
               {
//                   this->hide();
//                   status=ST_HIDE;
                   return true;
               }
               else if(mouseEvent->button() == Qt::RightButton)
               {
                   return true;
               }
           }
        else if(event->type() == QEvent::ContextMenu)
        {
            return false;
        }
        else if (event->type() == QEvent::WindowDeactivate)
        {
            //qDebug()<<"激活外部窗口";
            //this->hide();
            return true;
        } else if (event->type() == QEvent::StyleChange) {
        }
    }

//    if (!isActiveWindow())
//    {
//        activateWindow();
//    }
    return false;
}
