#include "schedule_item.h"
#include "schedulewidget.h"
#include <QWidget>
#include <QVBoxLayout>
#include <QHBoxLayout>
#include <QScreen>
#include <QLabel>
#include <QPainter>
#include <QAction>
#include <QMouseEvent>
#include <QGuiApplication>
#include <QMenu>
#include "calendardatabase.h"
#include <QColor>
#include <QGSettings>
#include "calendarcolor.h"
#include "colorarea.h"
#include "schedulewidget.h"
#define THEME_SCHAME "org.ukui.style"
#define COLOR_THEME "themeColor"
#ifndef Q_UNUSED

#define Q_UNUSED(x) (void)x;

#endif
schedule_item::schedule_item(QWidget *parent,bool ishasSchedule) : QWidget(parent) {
    // 初始化布局
    mainLayout = new QHBoxLayout(this);
    this->setMouseTracking(true);

    // 创建颜色区域
    colorArea = new ColorArea(this);
    colorArea->setFixedSize(4, 44); // 设置大小
    //实时监听主题变化

    mainLayout->addWidget(colorArea);
    hasSchdule = ishasSchedule;
    dateTimeLabel = new QLabel(this);
    dateTimeLabel->setStyleSheet("QLabel { font-size: 14px; }");
    dateTimeLabel->setMargin(0); // 设置内边距
    id = new QLabel(this);
    id->hide();
    eventLabel = new QLabel(this);
    eventLabel->setStyleSheet("QLabel { font-size: 14px; }");
    eventLabel->setMargin(0); // 设置内边距

    // 添加间距
    mainLayout->addSpacing(4);

    // 创建水平布局用于日期、时间和日程内容
    eventLayout = new QVBoxLayout();

    textLayout = new QVBoxLayout();

    textLayout->addWidget(dateTimeLabel);
    textLayout->addWidget(eventLabel);

    eventLayout->addLayout(textLayout);

    mainLayout->addLayout(eventLayout);

    // 添加间距
    eventLayout->addSpacing(3);
    if(ishasSchedule == false){
        colorArea->setFixedSize(4, 30);
        dateTimeLabel->setText("无日程");
        dateTimeLabel->setStyleSheet("QLabel { font-size: 20px; }");
        dateTimeLabel->setMargin(0); // 设置内边距
        eventLayout->addWidget(dateTimeLabel);
        mainLayout->addLayout(eventLayout);
    }
    // 设置布局
    setLayout(mainLayout);

}
void schedule_item::enterEvent(QEvent *event)
{

}
void schedule_item::setId(const QString &idd)
{
    id->setText(idd);
}
void schedule_item::setStartTime(const QString &startTime) {
//    QString star = QString(startTime);
    dateTimeLabel->setText("日程:"+startTime);
}
//设置 已经结束了的日程置灰 但是还没有做到日程ASC功能
void schedule_item::setEndTime(int end_hour,int end_minute,QDate date) {

    QString endTimeStr = QString("%1:%2").arg(end_hour, 2, 10, QChar('0')).arg(end_minute, 2, 10, QChar('0'));
    // 将结束时间字符串转换为 QDateTime 对象
    QDateTime endTimeObj = QDateTime(QDate::currentDate(), QTime(end_hour, end_minute), Qt::LocalTime);
    QDateTime currentTime = QDateTime::currentDateTime();
//    QDateTime endTimeObj = QDateTime::fromString(endTime, "hh:mm");
    if(date == QDate::currentDate()){
        if (currentTime > endTimeObj) {
            wasSchedule=true;
        }
    }
    if(wasSchedule){
        // 遍历所有子控件，查找并置灰标签
        eventLabel->setStyleSheet("color: gray; font-size: 14px;"); // 将标签文字置灰
        dateTimeLabel->setStyleSheet("color: grey ;font-size: 14px;");
    }


}
void schedule_item::adjustDate(QDate date)
{
    if(date < QDate::currentDate() ){
        wasSchedule = true;
    }else if (date > QDate::currentDate()){
        wasSchedule = false;
    }
}
//修改超过显示区域用省略号表示，并只有显示省略号的内容才显示悬浮框
void schedule_item::setDescription(const QString &description) {
    QFont font;
    font.setPointSize(14);
    QFontMetrics metrics(font);
    eventLabel->setFont(font);
    QString descriptionTrimmed = description.trimmed(); // 去除两端空白字符，确保准确比较长度
    QString elided = metrics.elidedText(descriptionTrimmed, Qt::ElideRight, 500);

    if (descriptionTrimmed.length() != elided.length()) {
        eventLabel->setToolTip(description);
    } else {
        eventLabel->setToolTip("");
    }
    eventLabel->setText(elided);
    // 设置全局的样式表
    eventLabel->setStyleSheet("QToolTip{color : black;background-color:white ; border:1px solid white;font-size: 14px; }"
                              "QLabel { font-size: 14px; }");
    if (wasSchedule) {
        eventLabel->setStyleSheet("QToolTip{color : black;background-color:white ; border:1px solid white;font-size: 14px; }"
                                  "QLabel { font-size: 14px;color: gray; }");
    }
}

void schedule_item::mousePressEvent(QMouseEvent *event) {
    if(event->button() == Qt::RightButton){
        if(dateTimeLabel->text().compare("无日程")!=0){
        this_id = getId();
        QMenu menu(this);
               // 添加“编辑日程”动作
               QAction *editAction = menu.addAction("编辑");
               // 添加“删除日程”动作
               QAction *deleteAction = menu.addAction("删除");
               // 显示菜单
               // 检查用户选择了哪个动作
              connect(editAction,&QAction::triggered,this,&schedule_item::slotEditSchedule);
              connect(deleteAction,&QAction::triggered,this,&schedule_item::slotDeleteSchedule);
              menu.exec(event->globalPos());
           }
    }
}
void schedule_item::mouseDoubleClickEvent(QMouseEvent *event)
{
    QString id = getId();
    if(event->button() == Qt::LeftButton){
        Q_EMIT ScheduleClick(id);
    }

}
void schedule_item::slotEditSchedule()
{
    Q_EMIT EditSchedule(this_id);
}
void schedule_item::slotDeleteSchedule()
{

    Q_EMIT updateWindow(this_id);



}
QString schedule_item::getId()
{
    return id->text();
}
